import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class MonitoringApi:
    def __init__(self, client: "Anypoint", log: logging.Logger):
        self._client = client
        self._log = log
        self._proxy_influx_db: str = ""
        self._influx_db: str = ""

    def sync(self):
        path = "/monitoring/api/sync"
        data = self._client.request(path)
        for p in data.get("proxies", []):
            if p["name"] == "influxdb":
                self._proxy_influx_db = p["id"]
            elif p["name"] == "influx_analytics":
                self._proxy_influx_analytics = p["id"]

    def authorize(self, org_id: str):
        path = "/accounts/api/authorize"
        data = {
            "namespace": "monitoringcenter",
            "action": "GET",
            "resources": [f"/organizations/{org_id}/admin", f"/organizations/{org_id}/viewer"],
        }
        return self._client.request(path, method="POST", body=data)

    def boot_data(self):
        path = "/monitoring/api/visualizer/api/bootdata"
        data = self._client.request(path)
        influx_data = data.get("Settings", {}).get("datasources", {}).get("influxdb", {})
        if influx_data:
            self._influx_db = influx_data["database"]
            self._proxy_influx_db = influx_data["id"]

    def get_applications(self, environment_id: str):
        # /monitoring/api/visualizer/environments/c22c6dfd-0257-42a5-b64e-6037910935e8/applications
        path = f"/monitoring/api/visualizer/environments/{environment_id}/applications"
        return self._client.request(path)

    def query(self, query_string: str) -> dict:
        if not self._proxy_influx_db:
            self.boot_data()
        path = f"/monitoring/api/visualizer/api/datasources/proxy/{self._proxy_influx_db}/query"
        return self._client.request(
            path, parameters={"db": self._influx_db, "q": query_string, "epoch": "ms"}
        )



from __future__ import annotations

from collections.abc import Generator
from datetime import datetime
from enum import Enum
from typing import TYPE_CHECKING

from anypoint.models.stats import DashboardStatistics, Statistic

if TYPE_CHECKING:
    from anypoint.api.application import ApplicationApi
    from anypoint.api.v2.application import ApplicationApiV2


class ApiVersion(Enum):
    V1 = "v1"
    V2 = "v2"


class Application:
    def __init__(
        self,
        raw_json,
        client: ApplicationApi | ApplicationApiV2,
        api_version: ApiVersion | None = ApiVersion.V1,
    ):
        self.domain: str = raw_json.get("domain")
        self.full_domain: str = raw_json.get("fullDomain")
        self.region: str = raw_json.get("region")
        self.status: str = raw_json.get("status")
        self.filename: str = raw_json.get("filename")  # Corrected typo: filename
        self.file_name = raw_json.get("filename")  # Duplicate, consider removing.
        self.href = raw_json.get("href")
        self.environment_id: str = raw_json.get(
            "environment_id"
        )  # You might not get this value from the provided responses

        if api_version == ApiVersion.V1:
            self.id: str = raw_json.get("id")
            self.mule_version: str = raw_json.get("muleVersion")
            self.deployment_group_id: str = raw_json.get("deploymentGroupId")
            self.remaining_workers: int = raw_json.get("remainingWorkerCount", 0)
            self.worker_type = raw_json.get("workerType")
            self.worker_count = raw_json.get("workers", 0)

        elif api_version == ApiVersion.V2:
            self.id: str = raw_json.get("versionId")  # Use versionId for V2
            mule_version_data = raw_json.get("muleVersion")
            self.mule_version: str = (
                mule_version_data.get("version") if mule_version_data else None
            )  # Nested access
            deployment_group = raw_json.get("deploymentGroup")
            self.deployment_group_id: str = deployment_group.get("id") if deployment_group else None
            workers_data = raw_json.get("workers")
            self.remaining_workers: float = (
                workers_data.get("remainingOrgWorkers", 0) if workers_data else 0
            )  # Can be float
            self.worker_type = workers_data.get("type", {}).get("name") if workers_data else None
            self.worker_count = workers_data.get("amount", 0) if workers_data else 0
            # No href in v2 response

        else:
            raise ValueError("Invalid API version")

        self.statistics = None
        self.worker_statistics: list[
            dict
        ] = []  # Replace list[WorkerStatistic] with actual type if you have it.
        self.workers: list[dict] = []  # Replace list[Worker] with actual type.

        self._data = raw_json
        self._api_client = client

        try:
            self.last_update_time: datetime = datetime.utcfromtimestamp(raw_json.get("lastUpdateTime") / 1000)
        except Exception:
            self.last_update_time: datetime = datetime.now()

    def __repr__(self):
        return f"Application({self.full_domain})"

    def __lt__(self, other):
        return self.domain < other.domain

    def get_statistics(self, date_from: datetime, date_to: datetime) -> Generator[Statistic, None, None]:
        return self._api_client.get_application_statistics(
            self.environment_id, self.domain, date_from, date_to
        )

    def get_dashboard_statistics(self, date_from: datetime, date_to: datetime) -> DashboardStatistics:
        return self._api_client.get_dashboard_statistics(self.environment_id, self.domain, date_from, date_to)

    def get_status(self):
        return self._api_client.get_application_status(self.environment_id, self.domain)

    def get_insights(self):
        return self._api_client.get_insights(self.environment_id, self.domain)


class ApplicationV2:
    def __init__(self, raw_json, _client: ApplicationApi):
        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.creation_date: datetime = datetime.utcfromtimestamp(raw_json.get("creationDate") / 1000)
        self.last_modified_date: datetime = datetime.utcfromtimestamp(raw_json.get("lastModifiedDate") / 1000)
        self.status: str = raw_json.get("status")
        self.application_status: str = raw_json.get("application", {}).get("status")
        self.current_runtime_version: str = raw_json.get("currentRuntimeVersion")
        self.last_successful_runtime_version: str = raw_json.get("lastSuccessfulRuntimeVersion")

        self._data = raw_json

    def __repr__(self):
        return f"ApplicationV2({self.name, self.status, self.application_status})"


class ApplicationV2Details:
    def __init__(self, raw_json, _client: ApplicationApi):
        self.environment_id = raw_json.get("environment_id")
        self.organization_id = raw_json.get("organization_id")

        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.domain = self.name
        self.full_domain: str = self.name

        self.public_url = (
            raw_json.get("target", {})
            .get("deploymentSettings", {})
            .get("http", {})
            .get("inbound", {})
            .get("publicUrl")
        )
        self.internal_url = (
            raw_json.get("target", {})
            .get("deploymentSettings", {})
            .get("http", {})
            .get("inbound", {})
            .get("internalUrl")
        )

        self.mule_version = raw_json.get("target", {}).get("deploymentSettings", {}).get("runtimeVersion")
        self.region = raw_json.get("target", {}).get("provider")
        self.file_name = raw_json.get("application", {}).get("ref", {}).get("artifactId")
        self.worker_count = raw_json.get("target", {}).get("replicas")
        self.remaining_workers = len(raw_json.get("replicas") or [])
        self.worker_type = raw_json.get("target", {}).get("instanceType")
        self.status = raw_json.get("application", {}).get("status")

        self.href = (
            f"https://anypoint.mulesoft.com/monitoring/#/builtin/{self.environment_id}/mc/{self.id}/overview"
        )

    def __repr__(self):
        return f"ApplicationV2Details({self.name, self.status})"



import logging

import requests

from anypoint.api.api_manager import ApiManagerApi
from anypoint.api.application import ApplicationApi
from anypoint.api.environment import EnvironmentApi
from anypoint.api.monitoring import MonitoringApi
from anypoint.api.mq import MQApi
from anypoint.api.organization import OrganizationApi
from anypoint.authentication import Authentication
from anypoint.http_client.client import HttpClient

_default_log = logging.getLogger(__name__)


class Anypoint:
    def __init__(
        self,
        authentication: Authentication,
        base_url: str = "https://anypoint.mulesoft.com",
        log: logging.Logger = _default_log,
        proxies: dict | None = None,
        http_timeout: int = 30,
    ):
        self._base_url = base_url
        self._authentication = authentication

        self.organizations = OrganizationApi(self, log)
        self.applications = ApplicationApi(self, log)
        self.environments = EnvironmentApi(self, log)
        self.monitoring = MonitoringApi(self, log)
        self.api_manager = ApiManagerApi(self, log)
        self.mq = MQApi(self, log)

        self._log = log
        self._access_token: str | None = None
        self._http_client = HttpClient(log, proxies, http_timeout=http_timeout)
        self._authentication.http_client = self._http_client

    def login(self):
        self._access_token = self._authentication.get_token(self._base_url)

    def me(self) -> dict:
        # TODO - Parse the response
        path = "/accounts/api/me"
        return self.request(path)

    def request(
        self,
        path: str,
        method: str = "GET",
        body: dict | None = None,
        headers: dict | None = None,
        parameters: dict | None = None,
        return_json=True,
    ) -> dict | requests.Response:
        url = f"{self._base_url}{path}"
        if not self._access_token and not url.endswith("/accounts/login"):
            self.login()
        if headers is None:
            headers = {}
        headers.update({"Authorization": f"Bearer {self._access_token}"})

        return self._http_client.request(url, method, body, headers, parameters, return_json)



from collections.abc import Generator
from typing import TYPE_CHECKING

from anypoint.models.application import Application, ApiVersion

if TYPE_CHECKING:
    from anypoint import Anypoint


class ApplicationApiV2:
    def __init__(self, anypoint: "Anypoint"):
        self._client = anypoint

    def get_applications(self, environment_id: str) -> Generator[Application, None, None]:
        path = "/cloudhub/api//v2/applications"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        data = self._client.request(path, headers=headers)

        for app in data:
            app["environment_id"] = environment_id
            yield Application(app, self, ApiVersion.V2)



import logging
from collections.abc import Generator
from datetime import datetime
from typing import TYPE_CHECKING

from anypoint import utils
from anypoint.models.application import Application, ApplicationV2, ApplicationV2Details
from anypoint.models.stats import DashboardStatistics, Statistic

if TYPE_CHECKING:
    from anypoint import Anypoint


class ApplicationApi:
    def __init__(self, anypoint: "Anypoint", log: logging.Logger):
        self._client = anypoint
        self._log = log

    def get_application_statistics(
        self, environment_id: str, app_domain: str, date_from: datetime, date_to: datetime
    ) -> Generator[Statistic, None, None]:
        path = f"/cloudhub/api/applications/{app_domain}/statistics"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        params = {
            "start": f"{int(date_from.timestamp() * 1000)}",
            "end": f"{int(date_to.timestamp() * 1000)}",
            "intervalCount": f"{(date_to - date_from).total_seconds() // 60:0.0f}",
        }
        data = self._client.request(path, parameters=params, headers=headers)
        for timestamp, value in data.items():
            yield Statistic(timestamp, value)

    def get_dashboard_statistics(
        self, environment_id: str, app_domain: str, date_from: datetime, date_to: datetime
    ) -> DashboardStatistics:
        path = f"/cloudhub/api/v2/applications/{app_domain}/dashboardStats"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        params = {
            "startDate": utils.date_to_str(date_from),
            "endDate": utils.date_to_str(date_to),
            "interval": "60000",
        }
        data = self._client.request(path, parameters=params, headers=headers)
        return DashboardStatistics(data)

    def get_applications(self, environment_id: str) -> Generator[Application, None, None]:
        path = "/cloudhub/api/applications"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        data = self._client.request(path, headers=headers)
        for app in data:
            app["environment_id"] = environment_id
            yield Application(app, self)

    def get_applications_v2(self, org_id: str, environment_id: str) -> Generator[ApplicationV2, None, None]:
        path = (
            f"/amc/application-manager/api/v2/organizations/{org_id}/"
            f"environments/{environment_id}/deployments"
        )
        data = self._client.request(path)
        for app in data["items"]:
            app["environment_id"] = environment_id
            app["organization_id"] = org_id
            yield ApplicationV2(app, self)

    def get_application_v2(
        self, org_id: str, environment_id: str, deployment_id: str
    ) -> ApplicationV2Details:
        path = (
            f"/amc/application-manager/api/v2/organizations/{org_id}/"
            f"environments/{environment_id}/deployments/{deployment_id}"
        )
        data = self._client.request(path)
        data["environment_id"] = environment_id
        data["organization_id"] = org_id
        return ApplicationV2Details(data, self)

    def get_insights(self, environment_id: str, app_domain: str):
        path = f"/api/v2/applications/{app_domain}/insight"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        return self._client.request(path, headers=headers)

    def get_application_status(self, environment_id: str, app_domain: str):
        path = f"/cloudhub/api/applications/{app_domain}/status"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        response = self._client.request(path, headers=headers, return_json=False)
        return response.text



import logging

import requests

# Disable urllib warnings
from requests.adapters import HTTPAdapter
from urllib3 import Retry

requests.packages.urllib3.disable_warnings()

REQUEST_TIMEOUT = 60


class CustomRetry(Retry):
    def get_backoff_time(self):
        return self.backoff_factor


class HttpClient:
    def __init__(self, log: logging.Logger, proxies: dict | None = None, http_timeout: int = REQUEST_TIMEOUT):
        self._log = log
        self._proxies = proxies
        self._session = requests.Session()
        self.http_timeout = http_timeout
        self._session.mount(
            "https://",
            HTTPAdapter(
                max_retries=CustomRetry(
                    total=3,
                    backoff_factor=0.5,
                    status_forcelist=[429, 500, 502, 503, 504],
                    allowed_methods=["TRACE", "PUT", "DELETE", "OPTIONS", "HEAD", "GET", "POST"],
                    raise_on_status=False,
                )
            ),
        )

    def request(
        self,
        url: str,
        method: str = "GET",
        body: dict | None = None,
        headers: dict | None = None,
        parameters: dict | None = None,
        return_json=True,
    ) -> dict | requests.Response:
        try:
            self._log.debug(f"Making request {method} {url} {parameters if parameters else ''}")
            r = self._session.request(
                method,
                url,
                json=body,
                params=parameters,
                headers=headers,
                timeout=(self.http_timeout, self.http_timeout),
                proxies=self._proxies,
                verify=False,
            )
            self._log.debug(f"Received response {method} {url} {parameters if parameters else ''}: {r}")
            if r.status_code >= 400:
                error_message = f"Received a bad response: {method} {url}: {r}: {r.content}"
                raise Exception(error_message)
            # self._log.debug(f"Response: {r.json()}")
            return r.json() if return_json else r
        except Exception as e:
            self._log.exception(f"Could not make request {method} {url}: {repr(e)}")
            raise e



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class VPNTunnel:
    def __init__(self, raw_json):
        self.accepted_route_count: int = raw_json.get("acceptedRouteCount")
        self.ike_versions = raw_json.get("ikeVersions", [])
        self.last_status_change: str = raw_json.get("lastStatusChange")
        self.local_external_ip_address: str = raw_json.get("localExternalIpAddress")
        self.local_ptp_ip_address: str = raw_json.get("localPtpIpAddress")
        self.phase1_dh_groups = raw_json.get("phase1DhGroups", [])
        self.phase1_encryption_algorithms = raw_json.get("phase1EncryptionAlgorithms", [])
        self.phase1_integrity_algorithms = raw_json.get("phase1IntegrityAlgorithms", [])
        self.phase2_dh_groups = raw_json.get("phase2DhGroups", [])
        self.phase2_encryption_algorithms = raw_json.get("phase2EncryptionAlgorithms", [])
        self.phase2_integrity_algorithms = raw_json.get("phase2IntegrityAlgorithms", [])
        self.psk: str = raw_json.get("psk")
        self.rekey_fuzz: int = raw_json.get("rekeyFuzz")
        self.rekey_margin_in_seconds: int = raw_json.get("rekeyMarginInSeconds")
        self.remote_ptp_ip_address: str = raw_json.get("remotePtpIpAddress")
        self.startup_action: str = raw_json.get("startupAction")
        self.status: str = raw_json.get("status")
        self.status_message: str = raw_json.get("statusMessage")
        self.vpn_connection_id: str = raw_json.get("vpnConnectionId")

        self._data = raw_json


class VPN:
    def __init__(self, raw_json):
        self.connection_id: str = raw_json.get("connectionId")
        self.connection_name: str = raw_json.get("connectionName")
        self.local_asn: int = raw_json.get("localAsn")
        self.name: str = raw_json.get("name")
        self.remote_asn: int = raw_json.get("remoteAsn")
        self.remote_ip_address: str = raw_json.get("remoteIpAddress")
        self.vpn_connection_status: str = raw_json.get("vpnConnectionStatus")
        self.vpn_id: str = raw_json.get("vpnId")
        self.vpn_tunnels = [VPNTunnel(tunnel) for tunnel in raw_json.get("vpnTunnels", [])]

        self._data = raw_json


class Connection:
    def __init__(self, raw_json):
        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.vpns = [VPN(vpn) for vpn in raw_json.get("vpns", [])]

        self._data = raw_json


class PrivateSpace:
    def __init__(self, raw_json, client: "Anypoint"):
        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.organization_id: str = raw_json.get("organizationId")
        self.region: str = raw_json.get("region")
        self.root_organization_id: str = raw_json.get("rootOrganizationId")
        self.status: str = raw_json.get("status")

        self._data = raw_json
        self._client = client

    def get_connections(self) -> list[Connection]:
        return self._client.organizations.get_private_space_connections(self.organization_id, self.id)
